# Generated by Django 3.2.11 on 2022-02-25 20:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0039_collectionremote_signed_only'),
    ]

    operations = [
        migrations.AddField(
            model_name='ansiblerepository',
            name='keyring',
            field=models.FilePathField(blank=True, path='/etc/pulp/certs/', recursive=True),
        ),
    ]
