# Generated by Django 4.2.16 on 2024-12-05 12:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0035_deprecation_content'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='ansiblecollectiondeprecated',
            unique_together={('namespace', 'name')},
        ),
        migrations.RemoveField(
            model_name='ansiblecollectiondeprecated',
            name='collection_id',
        ),
        migrations.RemoveField(
            model_name='ansiblecollectiondeprecated',
            name='repository_id',
        ),
        migrations.RemoveField(
            model_name='ansiblecollectiondeprecated',
            name='version_added_id',
        ),
    ]
