# Generated by Django 3.2.18 on 2023-03-20 13:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0102_add_domain_relations'),
        ('ansible', '0049_rbac_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='CrossRepositoryCollectionVersionIndex',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_deprecated', models.BooleanField()),
                ('is_signed', models.BooleanField()),
                ('is_highest', models.BooleanField()),
                ('collection_version', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ansible_crossrepositorycollectionversionindex', to='ansible.collectionversion')),
                ('namespace_metadata', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='ansible_crossrepositorycollectionversionindex', to='ansible.ansiblenamespacemetadata')),
                ('repository', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ansible_crossrepositorycollectionversionindex', to='ansible.ansiblerepository')),
                ('repository_version', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='ansible_crossrepositorycollectionversionindex', to='core.repositoryversion')),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
                'unique_together': {('repository', 'repository_version', 'collection_version')},
            },
        ),
    ]
