# Generated by Django 3.2.11 on 2022-01-10 21:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0038_collectionversionsignature'),
    ]

    operations = [
        migrations.AddField(
            model_name='collectionremote',
            name='signed_only',
            field=models.BooleanField(default=False),
        ),
    ]
