# Generated by Django 4.2.19 on 2025-05-21 12:04

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ansible", "0060_remove_collectionversion_unique_is_highest_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="collectionversion",
            name="new_tags",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.CharField(max_length=64, unique=True),
                db_column="tags",
                null=True,
                size=None,
            ),
        ),
    ]
