# Generated by Django 4.2.19 on 2025-04-15 16:27

from django.db import migrations, models
import django.db.models.deletion
import pulpcore.app.util


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0117_task_unblocked_at"),
        ("ansible", "0062_pivot_collectionversion_tags"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="ansiblecollectiondeprecated",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="ansiblenamespacemetadata",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="collectionversion",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="collectionversionmark",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="collectionversionsignature",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="role",
            unique_together=set(),
        ),
        migrations.AddField(
            model_name="ansiblecollectiondeprecated",
            name="_pulp_domain",
            field=models.ForeignKey(
                default=pulpcore.app.util.get_domain_pk,
                on_delete=django.db.models.deletion.PROTECT,
                to="core.domain",
            ),
        ),
        migrations.AddField(
            model_name="ansiblenamespacemetadata",
            name="_pulp_domain",
            field=models.ForeignKey(
                default=pulpcore.app.util.get_domain_pk,
                on_delete=django.db.models.deletion.PROTECT,
                to="core.domain",
            ),
        ),
        migrations.AddField(
            model_name="collectionversion",
            name="_pulp_domain",
            field=models.ForeignKey(
                default=pulpcore.app.util.get_domain_pk,
                on_delete=django.db.models.deletion.PROTECT,
                to="core.domain",
            ),
        ),
        migrations.AddField(
            model_name="collectionversionmark",
            name="_pulp_domain",
            field=models.ForeignKey(
                default=pulpcore.app.util.get_domain_pk,
                on_delete=django.db.models.deletion.PROTECT,
                to="core.domain",
            ),
        ),
        migrations.AddField(
            model_name="collectionversionsignature",
            name="_pulp_domain",
            field=models.ForeignKey(
                default=pulpcore.app.util.get_domain_pk,
                on_delete=django.db.models.deletion.PROTECT,
                to="core.domain",
            ),
        ),
        migrations.AddField(
            model_name="role",
            name="_pulp_domain",
            field=models.ForeignKey(
                default=pulpcore.app.util.get_domain_pk,
                on_delete=django.db.models.deletion.PROTECT,
                to="core.domain",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="ansiblecollectiondeprecated",
            unique_together={("namespace", "name", "_pulp_domain")},
        ),
        migrations.AlterUniqueTogether(
            name="ansiblenamespacemetadata",
            unique_together={("metadata_sha256", "_pulp_domain")},
        ),
        migrations.AlterUniqueTogether(
            name="collectionversion",
            unique_together={("sha256", "_pulp_domain")},
        ),
        migrations.AlterUniqueTogether(
            name="collectionversionmark",
            unique_together={("value", "marked_collection", "_pulp_domain")},
        ),
        migrations.AlterUniqueTogether(
            name="collectionversionsignature",
            unique_together={("pubkey_fingerprint", "signed_collection", "_pulp_domain")},
        ),
        migrations.AlterUniqueTogether(
            name="role",
            unique_together={("version", "name", "namespace", "_pulp_domain")},
        ),
    ]
