# Generated by Django 4.2.4 on 2023-09-15 09:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("ansible", "0054_split_collection_version_numbers"),
    ]

    operations = [
        migrations.RunSQL(
            # Custom roles are only allowed with postgreSQL 16.
            sql="CREATE COLLATION pulp_ansible_semver (provider = 'icu', locale = 'en-US-u-kn-true')",
            reverse_sql="DROP COLLATION pulp_ansible_semver",
        ),
        migrations.AlterField(
            model_name="collectionversion",
            name="version",
            field=models.CharField(db_collation="pulp_ansible_semver", max_length=128),
        ),
        migrations.AlterField(
            model_name="role",
            name="version",
            field=models.CharField(db_collation="pulp_ansible_semver", max_length=128),
        ),
    ]
